from pytest_matchers.matchers import Eq


def test_create():
    matcher = Eq(1)
    assert isinstance(matcher, Eq)


def test_matches():
    """What do you want to understand? It's completely unnecessary this matcher"""
    assert 1 == Eq(1)
    assert 1 != Eq(2)
    assert Eq(1) == 1


def test_repr():
    assert repr(Eq(1)) == "To be 1"
    assert repr(Eq("string")) == "To be 'string'"



from pytest_matchers.matchers import IsInstance, Or, Eq


def test_create():
    matcher = Or(Eq(1), Eq(2), Eq(3))
    assert isinstance(matcher, Or)
    matcher = Eq(1) | Eq(2) | Eq(3)
    assert isinstance(matcher, Or)


def test_repr():
    matcher = Or(Eq(1), IsInstance(int))
    assert repr(matcher) == "To be 1 or of 'int' instance"
    matcher = Or(Eq(1))
    assert repr(matcher) == "To be 1"
    matcher = IsInstance(int) | Eq(1)
    assert repr(matcher) == "Of 'int' instance or to be 1"


def test_matches():
    assert 1 == Eq(1) | Eq(1)
    assert 1 == Eq(1) | Eq(2)
    assert 1 == Eq(2) | Eq(1)
    assert 1 != Eq(2) | Eq(3)
    assert 1 != Eq(3) | Eq(2)
    assert Eq(1) | Eq(1) == Eq(1)


def test_matches_exact_value():
    matcher = Or(IsInstance(str), 8)
    assert matcher == 8
    assert matcher == 8.0
    assert matcher != 1
    assert matcher == "string"
    assert matcher != ["string"]



from pytest_matchers.matchers import StartsWith


def test_create():
    matcher = StartsWith("a")
    assert isinstance(matcher, StartsWith)
    matcher = StartsWith([1, 2])
    assert isinstance(matcher, StartsWith)


def test_repr():
    matcher = StartsWith("a")
    assert repr(matcher) == "To start with 'a'"
    matcher = StartsWith([1, 2])
    assert repr(matcher) == "To start with [1, 2]"


def test_concatenated_repr():
    matcher = StartsWith("a")
    assert matcher.concatenated_repr() == "starting with 'a'"
    matcher = StartsWith([1, 2])
    assert matcher.concatenated_repr() == "starting with [1, 2]"


def test_matches_string():
    matcher = StartsWith("ab")
    assert matcher == "ab"
    assert matcher == "abc"
    assert matcher != "dabc"
    assert matcher != ["x", "ab", "y"]
    assert matcher != "ac"
    assert matcher != ["abc"]
    assert matcher != 20
    assert matcher != ["ab", "x"]


def test_matches_list():
    matcher = StartsWith([1, 3])
    assert matcher == [1, 3, 2]
    assert matcher == [1, 3]
    assert matcher != [1, 20, 1, 3, 30]
    assert matcher != [10, 20, 30]
    assert matcher != "string"
    assert matcher != 20



from unittest.mock import MagicMock

from pytest_matchers import is_list, is_string
from pytest_matchers.matchers.has_attribute import HasAttribute, has_attribute_matcher


class WithAttributeMethod:
    @staticmethod
    def attribute_name():  # pragma: no cover
        return "expected_value"


def test_create():
    matcher = HasAttribute("attribute_name")
    assert isinstance(matcher, HasAttribute)
    matcher = HasAttribute("attribute_name", "expected_value")
    assert isinstance(matcher, HasAttribute)


def test_repr():
    matcher = HasAttribute("attribute_name")
    assert repr(matcher) == "To have attribute 'attribute_name'"
    matcher = HasAttribute("attribute_name", "expected_value")
    assert repr(matcher) == "To have attribute 'attribute_name' and to be 'expected_value'"
    matcher = HasAttribute("attribute_name", 42)
    assert repr(matcher) == "To have attribute 'attribute_name' and to be 42"
    matcher = HasAttribute("attribute_name", is_string())
    assert repr(matcher) == "To have attribute 'attribute_name' and to be string"


def test_concatenated_repr():
    matcher = HasAttribute("attribute_name")
    assert matcher.concatenated_repr() == "with 'attribute_name'"
    matcher = HasAttribute("attribute_name", "expected_value")
    assert matcher.concatenated_repr() == "with 'attribute_name' being 'expected_value'"
    matcher = HasAttribute("attribute_name", 42)
    assert matcher.concatenated_repr() == "with 'attribute_name' being 42"
    matcher = HasAttribute("attribute_name", is_string())
    assert matcher.concatenated_repr() == "with 'attribute_name' expecting to be string"


def test_matches():
    mock = MagicMock(attribute_name="expected_value")
    matcher = HasAttribute("attribute_name")
    assert matcher == mock
    assert matcher == WithAttributeMethod
    assert matcher != 3
    assert matcher != "string"
    matcher = HasAttribute("attribute_name", "expected_value")
    assert matcher == mock
    assert matcher != WithAttributeMethod
    assert matcher != WithAttributeMethod()


def test_matches_with_expect_value_as_matcher():
    int_list_mock = MagicMock(attribute_name=[1, 2, 3])
    str_mock = MagicMock(attribute_name="string")
    matcher = HasAttribute("attribute_name", is_list(int))
    assert matcher == int_list_mock
    assert matcher != str_mock
    assert matcher != [1, 2, 3]
    matcher = HasAttribute("attribute_name", is_list(int, length=3))
    assert matcher == int_list_mock
    assert matcher != str_mock
    matcher = HasAttribute("attribute_name", is_list(int, length=2))
    assert matcher != int_list_mock
    assert matcher != str_mock
    matcher = HasAttribute("attribute_name", is_string())
    assert matcher != int_list_mock
    assert matcher == str_mock


def test_has_attribute_matcher():
    matcher = has_attribute_matcher("attribute_name", None)
    assert isinstance(matcher, HasAttribute)
    matcher = has_attribute_matcher("attribute_name", None, value_needed=True)
    assert matcher is None
    matcher = has_attribute_matcher("attribute_name", "expected_value")
    assert isinstance(matcher, HasAttribute)
    matcher = has_attribute_matcher("attribute_name", "expected_value", value_needed=True)
    assert isinstance(matcher, HasAttribute)
    matcher = has_attribute_matcher(None, None)
    assert matcher is None



from pytest_matchers.matchers import IsInstance


def test_create():
    matcher = IsInstance(str)
    assert isinstance(matcher, IsInstance)


def test_repr():
    matcher = IsInstance(str)
    assert repr(matcher) == "To be instance of 'str'"
    matcher = IsInstance(object)
    assert repr(matcher) == "To be instance of 'object'"


def test_concatenated_repr():
    matcher = IsInstance(str)
    assert matcher.concatenated_repr() == "of 'str' instance"
    matcher = IsInstance(object)
    assert matcher.concatenated_repr() == "of 'object' instance"


def test_matches():
    assert "string" == IsInstance(str)
    assert IsInstance(str) == "string"
    assert "string" == IsInstance(object)
    assert "string" != IsInstance(int)
    assert 3 == IsInstance(int)
    assert 3 != IsInstance(str)
    assert "string" != IsInstance(int)



from pytest_matchers.matchers import Eq, IsInstance, Not


def test_create():
    matcher = Not(Eq(1))
    assert isinstance(matcher, Not)
    matcher = ~Eq(1)
    assert isinstance(matcher, Not)
    matcher = Not(6)
    assert isinstance(matcher, Not)


def test_repr():
    matcher = Not(Eq(1))
    assert repr(matcher) == "Not to be 1"
    matcher = ~Eq(1)
    assert repr(matcher) == "Not to be 1"
    matcher = Not(6)
    assert repr(matcher) == "Not to be 6"
    matcher = Not(IsInstance(int))
    assert repr(matcher) == "Not of 'int' instance"
    matcher = Not(Not(IsInstance(int)))
    assert repr(matcher) == repr(IsInstance(int))


def test_invert():
    matcher = ~Not(IsInstance(int))
    assert isinstance(matcher, IsInstance)
    matcher = Not(Not(IsInstance(int)))
    assert isinstance(matcher, IsInstance)
    matcher = Not(Not(1))
    assert isinstance(matcher, Eq)


def test_matches():
    matcher = Not(Eq(1))
    assert 1 != matcher
    assert 2 == matcher
    matcher = ~IsInstance(str)
    assert 1 == matcher
    assert "string" != matcher



import pytest

from pytest_matchers.matchers import Between


def test_create():
    matcher = Between(1, 2)
    assert isinstance(matcher, Between)
    matcher = Between(1, None, inclusive=False)
    assert isinstance(matcher, Between)
    matcher = Between(1, 2, min_inclusive=False)
    assert isinstance(matcher, Between)
    matcher = Between(1, 2, max_inclusive=False)
    assert isinstance(matcher, Between)
    with pytest.raises(
        ValueError,
        match="Cannot specify inclusive and min_inclusive or max_inclusive",
    ):
        Between(1, 2, inclusive=True, min_inclusive=True)
    with pytest.raises(
        ValueError,
        match="At least one of min or max must be specified",
    ):
        Between(None, None)


def test_repr():
    matcher = Between(1, 2)
    assert repr(matcher) == "To be between 1 and 2"
    matcher = Between(1, 2, inclusive=False)
    assert repr(matcher) == "To be between 1 and 2 exclusive"
    matcher = Between(1, 2, min_inclusive=False)
    assert repr(matcher) == "To be greater than 1 and lower or equal than 2"
    matcher = Between(1, 2, max_inclusive=False)
    assert repr(matcher) == "To be greater or equal than 1 and lower than 2"
    matcher = Between(1, None)
    assert repr(matcher) == "To be greater or equal than 1"
    matcher = Between(None, 2, max_inclusive=False)
    assert repr(matcher) == "To be lower than 2"


def test_concatenated_repr():
    matcher = Between(1, 2)
    assert matcher.concatenated_repr() == "between 1 and 2"
    matcher = Between(1, 2, inclusive=False)
    assert matcher.concatenated_repr() == "between 1 and 2 exclusive"
    matcher = Between(1, 2, min_inclusive=False)
    assert matcher.concatenated_repr() == "greater than 1 and lower or equal than 2"
    matcher = Between(1, 2, max_inclusive=False)
    assert matcher.concatenated_repr() == "greater or equal than 1 and lower than 2"
    matcher = Between(1, None)
    assert matcher.concatenated_repr() == "greater or equal than 1"
    matcher = Between(None, 2, max_inclusive=False)
    assert matcher.concatenated_repr() == "lower than 2"


def test_matches_float_inclusive():
    matcher = Between(1, 2)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher == 2
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_string_inclusive():
    matcher = Between("a", "d")
    assert matcher == "a"
    assert matcher == "c"
    assert matcher == "d"
    assert matcher == "ca"
    assert matcher != "da"
    assert matcher != "z"
    assert matcher != 20
    assert matcher != ["a", "b"]


def test_matches_float_exclusive():
    matcher = Between(1, 2, inclusive=False)
    assert matcher == 1.5
    assert matcher != 1
    assert matcher != 2
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_float_min_exclusive():
    matcher = Between(1, 2, min_inclusive=False)
    assert matcher == 1.5
    assert matcher == 2
    assert matcher != 1
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_float_max_exclusive():
    matcher = Between(1, 2, max_inclusive=False)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher != 2
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_without_min():
    matcher = Between(None, 2)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher == 2
    assert matcher != 2.1
    assert matcher != "string"


def test_matches_without_max():
    matcher = Between(1, None)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher == 2
    assert matcher == 30000
    assert matcher != 0
    assert matcher != "string"



from pytest_matchers.matchers import And, Eq, IsInstance


def test_create():
    matcher = And(Eq(1), Eq(2), Eq(3))
    assert isinstance(matcher, And)
    matcher = Eq(1) & Eq(2) & Eq(3)
    assert isinstance(matcher, And)


def test_repr():
    matcher = And(Eq(1), IsInstance(int))
    assert repr(matcher) == "To be 1 and of 'int' instance"
    matcher = And(Eq(1))
    assert repr(matcher) == "To be 1"
    matcher = IsInstance(int) & Eq(1)
    assert repr(matcher) == "Of 'int' instance and to be 1"


def test_matches():
    assert 1 == Eq(1) & Eq(1)
    assert 1 != Eq(1) & Eq(2)
    assert 1 != Eq(2) & Eq(1)
    assert Eq(1) & Eq(1) == Eq(1)


def test_matches_exact_value():
    matcher = And(IsInstance(float), 8)
    assert matcher == 8.0
    assert matcher != 8
    assert matcher != 1
    assert matcher != "string"
    assert matcher != ["string"]



import pytest

from pytest_matchers.matchers import IsNumber


def test_create():
    matcher = IsNumber()
    assert isinstance(matcher, IsNumber)
    matcher = IsNumber(int)
    assert isinstance(matcher, IsNumber)
    matcher = IsNumber(min_value=1, max_value=2)
    assert isinstance(matcher, IsNumber)
    matcher = IsNumber(min_value=1, max_value=2, min_inclusive=False)
    assert isinstance(matcher, IsNumber)
    with pytest.raises(
        ValueError,
        match="Cannot specify inclusive and min_inclusive or max_inclusive",
    ):
        IsNumber(min_value=1, max_value=2, inclusive=True, min_inclusive=True)


def test_repr():
    matcher = IsNumber()
    assert repr(matcher) == "To be a number"
    matcher = IsNumber(int)
    assert repr(matcher) == "To be a number of 'int' instance"
    matcher = IsNumber(min_value=1, max_value=2)
    assert repr(matcher) == "To be a number between 1 and 2"
    matcher = IsNumber(min_value=1, max_value=2, inclusive=False)
    assert repr(matcher) == "To be a number between 1 and 2 exclusive"
    matcher = IsNumber(min_value=1, max_value=2, min_inclusive=False)
    assert repr(matcher) == "To be a number greater than 1 and lower or equal than 2"
    matcher = IsNumber(min_value=1, max_value=2, max_inclusive=False)
    assert repr(matcher) == "To be a number greater or equal than 1 and lower than 2"
    matcher = IsNumber(float, min_value=1, max_value=2)
    assert repr(matcher) == "To be a number of 'float' instance and between 1 and 2"


def test_matches_number():
    matcher = IsNumber()
    assert matcher == 20
    assert matcher == 20.0
    assert matcher != "string"
    assert matcher != ["string"]


def test_matches_type():
    matcher = IsNumber(int)
    assert matcher == 20
    assert matcher != 20.0
    assert matcher != "string"
    assert matcher != ["string"]


def test_matches_limit_inclusive():
    matcher = IsNumber(min_value=1, max_value=2)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher == 2
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_limit_exclusive():
    matcher = IsNumber(min_value=1, max_value=2, inclusive=False)
    assert matcher == 1.5
    assert matcher != 1
    assert matcher != 2
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_min_exclusive():
    matcher = IsNumber(min_value=1, max_value=2, min_inclusive=False)
    assert matcher == 1.5
    assert matcher == 2
    assert matcher != 1
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_max_exclusive():
    matcher = IsNumber(min_value=1, max_value=2, max_inclusive=False)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher != 2
    assert matcher != 0
    assert matcher != 3
    assert matcher != "string"


def test_matches_without_max():
    matcher = IsNumber(min_value=1)
    assert matcher == 1
    assert matcher == 1.5
    assert matcher == 3
    assert matcher != 0.99
    assert matcher != "string"
    assert matcher != ["string"]
    assert matcher != [1, 2]



from datetime import datetime

from pytest_matchers.matchers import Datetime


def test_create():
    matcher = Datetime()
    assert isinstance(matcher, Datetime)
    matcher = Datetime(min_value=datetime(2021, 1, 1))
    assert isinstance(matcher, Datetime)
    matcher = Datetime(max_value=datetime(2021, 1, 1))
    assert isinstance(matcher, Datetime)
    matcher = Datetime(year=2021)
    assert isinstance(matcher, Datetime)
    matcher = Datetime(month=1)
    assert isinstance(matcher, Datetime)
    matcher = Datetime(day=1)
    assert isinstance(matcher, Datetime)
    matcher = Datetime(
        min_value=datetime(2021, 1, 1),
        max_value=datetime(2022, 10, 1),
        year=2021,
        month=1,
        day=1,
        hour=10,
        minute=30,
        second=45,
    )
    assert isinstance(matcher, Datetime)


def test_repr():
    matcher = Datetime()
    assert repr(matcher) == "To be a datetime"
    matcher = Datetime(min_value=datetime(2021, 1, 1), max_value=datetime(2021, 1, 2))
    assert repr(matcher) == "To be a datetime between 2021-01-01 00:00:00 and 2021-01-02 00:00:00"
    matcher = Datetime(min_value=datetime(2021, 1, 1))
    assert repr(matcher) == "To be a datetime greater or equal than 2021-01-01 00:00:00"
    matcher = Datetime(max_value=datetime(2021, 1, 1))
    assert repr(matcher) == "To be a datetime lower or equal than 2021-01-01 00:00:00"
    matcher = Datetime(year=2021)
    assert repr(matcher) == "To be a datetime with 'year' being 2021"
    matcher = Datetime(month=1)
    assert repr(matcher) == "To be a datetime with 'month' being 1"
    matcher = Datetime(day=1)
    assert repr(matcher) == "To be a datetime with 'day' being 1"
    matcher = Datetime(
        min_value=datetime(2021, 1, 1),
        max_value=datetime(2022, 10, 1),
        year=2021,
        month=1,
        day=1,
        hour=10,
        minute=30,
        second=45,
    )
    assert repr(matcher) == (
        "To be a datetime between 2021-01-01 00:00:00 and 2022-10-01 00:00:00 "
        "and with 'year' being 2021 "
        "and with 'month' being 1 "
        "and with 'day' being 1 "
        "and with 'hour' being 10 "
        "and with 'minute' being 30 "
        "and with 'second' being 45"
    )


def test_matches_instance():
    matcher = Datetime()
    assert matcher == datetime(2021, 1, 1)
    assert matcher == datetime(2021, 1, 1, 10, 30, 45)
    assert matcher != 2021
    assert matcher != "2021-01-01"


def test_matches_limit():
    matcher = Datetime(min_value=datetime(2021, 1, 1), max_value=datetime(2021, 1, 2))
    assert matcher == datetime(2021, 1, 1, 10, 30, 45)
    assert matcher == datetime(2021, 1, 2)
    assert matcher == datetime(2021, 1, 1)
    assert matcher != datetime(2021, 1, 2, 10, 30, 45)
    assert matcher != datetime(2021, 1, 2, 10, 30, 46)
    assert matcher != datetime(2021, 1, 3)
    matcher = Datetime(min_value=datetime(2021, 1, 1))
    assert matcher == datetime(2021, 1, 1)
    assert matcher == datetime(2021, 1, 2)
    assert matcher == datetime(2090, 1, 1)
    assert matcher != datetime(2020, 12, 31)
    matcher = Datetime(max_value=datetime(2021, 1, 1))
    assert matcher == datetime(2021, 1, 1)
    assert matcher == datetime(2020, 12, 31)
    assert matcher == datetime(1020, 1, 1)
    assert matcher != datetime(2021, 1, 2)


def test_matches_year():
    matcher = Datetime(year=2021)
    assert matcher == datetime(2021, 1, 1)
    assert matcher == datetime(2021, 12, 31)
    assert matcher != datetime(2020, 12, 31)
    assert matcher != datetime(2022, 1, 1)


def test_matches_month():
    matcher = Datetime(month=1)
    assert matcher == datetime(2021, 1, 1)
    assert matcher == datetime(2021, 1, 31)
    assert matcher != datetime(2021, 2, 1)
    assert matcher != datetime(2021, 12, 31)


def test_matches_day():
    matcher = Datetime(day=1)
    assert matcher == datetime(2021, 1, 1)
    assert matcher == datetime(2024, 10, 1, 10, 30, 45)
    assert matcher != datetime(2021, 1, 31)
    assert matcher != datetime(2021, 2, 28)


def test_matches_hour():
    matcher = Datetime(hour=10)
    assert matcher == datetime(2021, 1, 1, 10)
    assert matcher == datetime(2021, 1, 1, 10, 30)
    assert matcher != datetime(2021, 1, 1, 9)
    assert matcher != datetime(2021, 1, 1, 11)


def test_matches_minute():
    matcher = Datetime(minute=30)
    assert matcher == datetime(2021, 1, 1, 10, 30)
    assert matcher == datetime(2021, 1, 1, 10, 30, 45)
    assert matcher != datetime(2021, 1, 1, 10, 29)
    assert matcher != datetime(2021, 1, 1, 10, 31)


def test_matches_second():
    matcher = Datetime(second=45)
    assert matcher == datetime(2021, 1, 1, 10, 30, 45)
    assert matcher == datetime(2024, 10, 11, 21, 15, 45, 1000)
    assert matcher != datetime(2021, 1, 1, 10, 30, 44)
    assert matcher != datetime(2021, 1, 1, 10, 30, 46)
    assert matcher != datetime(2021, 1, 1, 10, 30)



from pytest_matchers.matchers import Contains


def test_create():
    matcher = Contains(20)
    assert isinstance(matcher, Contains)


def test_repr():
    matcher = Contains(20)
    assert repr(matcher) == "To contain 20"
    matcher = Contains("ab")
    assert repr(matcher) == "To contain 'ab'"


def test_concatenated_repr():
    matcher = Contains(20)
    assert matcher.concatenated_repr() == "containing 20"
    matcher = Contains("ab")
    assert matcher.concatenated_repr() == "containing 'ab'"


def test_matches_list():
    matcher = Contains(20)
    assert matcher == [10, 20, 30]
    assert matcher != [10, 21]
    assert matcher != "string"
    assert matcher != 20


def test_matches_string():
    matcher = Contains("ab")
    assert matcher == "ab"
    assert matcher == "abc"
    assert matcher == "dabc"
    assert matcher == ["x", "ab", "y"]
    assert matcher != "ac"
    assert matcher != ["abc"]
    assert matcher != 20



import pytest

from pytest_matchers.matchers import IsString


def test_create():
    matcher = IsString()
    assert isinstance(matcher, IsString)
    matcher = IsString(starts_with="a", ends_with="b", contains="c")
    assert isinstance(matcher, IsString)
    matcher = IsString(length=1)
    assert isinstance(matcher, IsString)

    with pytest.raises(
        ValueError,
        match="Cannot specify length with min_length or max_length",
    ):
        IsString(min_length=1, max_length=4, length=2)


def test_repr():
    matcher = IsString()
    assert repr(matcher) == "To be string"
    matcher = IsString(contains="ab")
    assert repr(matcher) == "To be string containing 'ab'"
    matcher = IsString(starts_with="ab")
    assert repr(matcher) == "To be string starting with 'ab'"
    matcher = IsString(ends_with="bc")
    assert repr(matcher) == "To be string ending with 'bc'"
    matcher = IsString(length=1)
    assert repr(matcher) == "To be string with length of 1"
    matcher = IsString(min_length=1, max_length=3)
    assert repr(matcher) == "To be string with length between 1 and 3"
    matcher = IsString(
        contains="ab",
        starts_with="ab",
        ends_with="bc",
        min_length=1,
        max_length=8,
    )
    assert (
        repr(matcher) == "To be string with length between 1 and 8 and containing 'ab'"
        " and starting with 'ab' and ending with 'bc'"
    )


def test_matches_type():
    matcher = IsString()
    assert matcher == "string"
    assert matcher != 20
    assert matcher != ["string"]


def test_matches_exact_length():
    matcher = IsString(length=1)
    assert matcher == "a"
    assert matcher != ""
    assert matcher != "ab"
    assert matcher != "string"


def test_matches_min_and_max_length():
    matcher = IsString(min_length=1, max_length=3)
    assert matcher == "a"
    assert matcher == "ab"
    assert matcher == "abc"
    assert matcher != ""
    assert matcher != "abcd"


def test_matches_contains():
    matcher = IsString(contains="ab")
    assert matcher == "ab"
    assert matcher == "abc"
    assert matcher == "dabc"
    assert matcher != ["x", "ab", "y"]
    assert matcher != "ac"
    assert matcher != ["abc"]
    assert matcher != 20


def test_matches_starts_with():
    matcher = IsString(starts_with="ab")
    assert matcher == "ab"
    assert matcher == "abc"
    assert matcher != "dabc"
    assert matcher != ["x", "ab", "y"]
    assert matcher != "ac"
    assert matcher != ["abc"]
    assert matcher != 20


def test_matches_ends_with():
    matcher = IsString(ends_with="bc")
    assert matcher == "abc"
    assert matcher == "bc"
    assert matcher != "dabcx"
    assert matcher != ["x", "bc", "y"]
    assert matcher != "ac"
    assert matcher != ["abc"]
    assert matcher != 20
    assert matcher != ["x", "bc"]



import pytest

from pytest_matchers.matchers import Length


def test_create():
    matcher = Length()
    assert isinstance(matcher, Length)
    matcher = Length(length=1)
    assert isinstance(matcher, Length)
    matcher = Length(min_length=1, max_length=4)
    assert isinstance(matcher, Length)

    with pytest.raises(
        ValueError,
        match="Cannot specify length with min_length or max_length",
    ):
        Length(length=1, min_length=1)


def test_repr():
    matcher = Length()
    assert repr(matcher) == ""
    matcher = Length(length=1)
    assert repr(matcher) == "To have length of 1"
    matcher = Length(min_length=1, max_length=3)
    assert repr(matcher) == "To have length between 1 and 3"
    matcher = Length(min_length=1)
    assert repr(matcher) == "To have length greater or equal than 1"
    matcher = Length(max_length=3)
    assert repr(matcher) == "To have length lower or equal than 3"


def test_concatenated_repr():
    matcher = Length()
    assert matcher.concatenated_repr() == ""
    matcher = Length(length=1)
    assert matcher.concatenated_repr() == "with length of 1"
    matcher = Length(min_length=1, max_length=3)
    assert matcher.concatenated_repr() == "with length between 1 and 3"
    matcher = Length(min_length=1)
    assert matcher.concatenated_repr() == "with length greater or equal than 1"
    matcher = Length(max_length=3)
    assert matcher.concatenated_repr() == "with length lower or equal than 3"


def test_matches_exact_length():
    matcher = Length(length=1)
    assert matcher.matches([1])
    assert matcher.matches("a")
    assert matcher.matches({"key": "value"})
    assert not matcher.matches([])
    assert not matcher.matches([1, 2])
    assert not matcher.matches("string")


def test_matches_min_and_max_length():
    matcher = Length(min_length=1, max_length=3)
    assert matcher.matches([1])
    assert matcher.matches([1, 2])
    assert matcher.matches([1, 2, 3])
    assert not matcher.matches([])
    assert not matcher.matches([1, 2, 3, 4])


def test_matches_for_value_without_len():
    matcher = Length(min_length=1, max_length=3)
    assert not matcher.matches(1)


def test_matches_without_limits():
    matcher = Length()
    assert matcher.matches([])
    assert matcher.matches([1])
    assert matcher.matches([1, 2])
    assert not matcher.matches(1)



from pytest_matchers.matchers import Anything


def test_create():
    matcher = Anything()
    assert isinstance(matcher, Anything)


def test_repr():
    matcher = Anything()
    assert repr(matcher) == "To be anything"


def test_matches():
    matcher = Anything()
    assert matcher == 1
    assert matcher == "string"
    assert matcher == ["string"]
    assert matcher == {"key": "value"}
    assert matcher == object
    assert matcher == len  # pylint: disable=comparison-with-callable



from datetime import datetime
from random import random
from unittest.mock import MagicMock

from pytest_matchers import is_datetime, is_datetime_string, is_instance, is_list, is_number
from pytest_matchers import between, has_attribute


def test_random_number():
    assert random() == is_number(min_value=0, max_value=1)
    assert random() == between(0, 1)


def test_dict_comparison():
    value = {
        "string": "string",
        "int": 3,
        "list": [1, 2, 3],
    }
    assert value == {
        "string": is_instance(str),
        "int": is_instance(int),
        "list": is_list(int, length=3),
    }
    assert value != {"string": is_instance(str), "int": is_instance(int)}


def test_mock_log_exception():
    def _logging_function(logging):
        try:
            raise ValueError("This is a test exception")
        except ValueError as error:
            logging.exception("Caught exception: %s", error)

    mock_logging = MagicMock()
    _logging_function(mock_logging)
    mock_logging.exception.assert_called_with(
        "Caught exception: %s",
        is_instance(ValueError) & has_attribute("args", ("This is a test exception",)),
    )


def test_datetime_comparison():
    def _datetime_function():
        return {
            "data": "some data",
            "created_at": datetime.now(),
            "created_at_string": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }

    expected_value = {
        "data": "some data",
        "created_at": is_datetime(min_value=datetime.now()),
        "created_at_string": is_datetime_string("%Y-%m-%d %H:%M:%S", min_value=datetime.now()),
    }
    first_data = _datetime_function()
    assert first_data == expected_value
    second_data = _datetime_function()
    assert second_data == expected_value
    assert first_data != second_data



from datetime import datetime
from unittest.mock import MagicMock

from pytest_matchers import (
    between,
    has_attribute,
    is_instance,
    is_list,
    is_number,
    is_string,
    one_of,
    is_datetime,
    is_datetime_string,
)


def test_is_instance():
    assert "string" == is_instance(str)
    assert 3 != is_instance(str)


def test_is_list():
    assert [1, 2, 3] == is_list(int, length=3)
    assert [1, 2, 3] != is_list(str, length=3)
    assert [1, 2, 3] != is_list(int, length=2)
    assert [1, 2, 3] == is_list(int, min_length=2)
    assert [1, 2, 3] != is_list(int, min_length=4)
    assert [1, 2, 3] == is_list(int, max_length=4)
    assert [1, 2, 3] != is_list(int, max_length=2)
    assert ["abc"] == is_list()


def test_is_string():
    assert "string" == is_string()
    assert "string" == is_string(starts_with="s")
    assert "string" == is_string(ends_with="g")
    assert "string" == is_string(contains="ri")
    assert "string" == is_string(length=6)
    assert "string" == is_string(min_length=6)
    assert "string" == is_string(max_length=6)
    assert 3 != is_string()
    assert "string" != is_string(starts_with="a")
    assert "string" != is_string(ends_with="a")
    assert "string" != is_string(contains="a")
    assert "string" != is_string(length=5)
    assert "string" != is_string(min_length=7)
    assert "string" != is_string(max_length=5)


def test_is_number():
    assert 3 == is_number()
    assert "3" == is_number()
    assert 3 == is_number(int)
    assert 3 == is_number(min_value=2)
    assert 3 == is_number(max_value=4)
    assert 3 == is_number(min_value=2, max_value=4)
    assert "text" != is_number()
    assert "3.5" != is_number(int)
    assert 3 != is_number(float)
    assert 3 != is_number(min_value=4)
    assert 3 != is_number(max_value=2)
    assert 3 != is_number(min_value=4, max_value=2)
    assert 3 != is_number(min_value=4, max_value=7)


def test_one_of():
    assert 1 == one_of(1, 4)
    assert 4 == one_of(1, 4)
    assert 1 != one_of(2, 3)
    assert 4 == one_of(is_instance(str), is_instance(int))


def test_has_attribute():
    mock = MagicMock(attribute=30)
    assert "string" == has_attribute("lower")
    assert "string" != has_attribute("lower", "string")
    assert "string" != has_attribute("attribute")
    assert mock == has_attribute("attribute")
    assert mock == has_attribute("attribute", 30)
    assert mock != has_attribute("attribute", 40)


def test_between():
    assert 2 == between(1, 3)
    assert 1 == between(1, 3)
    assert 2 == between(1, 3, inclusive=False)
    assert 3 != between(1, 3, inclusive=False)
    assert 1 == between(1, 3, max_inclusive=False)
    assert 3 != between(1, 3, max_inclusive=False)
    assert 1 != between(1, 3, min_inclusive=False)
    assert 3 == between(1, 3, min_inclusive=False)
    assert "c" == between("a", "d")


def test_is_datetime():
    assert datetime(2021, 1, 1) == is_datetime()
    assert datetime(2021, 1, 1) == is_datetime(year=2021, month=1, day=1)
    assert datetime(2021, 1, 1) != is_datetime(year=2021, month=1, day=2)
    assert datetime(2021, 1, 2) == is_datetime(min_value=datetime(2021, 1, 1))
    assert datetime(2021, 1, 1) == is_datetime(max_value=datetime(2021, 1, 2))
    assert datetime(2021, 1, 1, 20, 15, 15) == is_datetime(
        min_value=datetime(2021, 1, 1),
        max_value=datetime(2021, 1, 2),
    )
    assert datetime(2021, 1, 1, 20, 14, 15) == is_datetime(
        year=2021,
        month=1,
        day=1,
        hour=20,
        minute=14,
        second=15,
    )


def test_is_datetime_string():
    assert "2021-01-01" != is_datetime_string("erroneous_format")
    assert "2021-01-01" == is_datetime_string("%Y-%m-%d")
    assert "2021-01-01" == is_datetime_string("%Y-%m-%d")
    assert "2021-01-01" != is_datetime_string("%Y-%m-%d %H:%M:%S")
    assert "2021-01-01" == is_datetime_string("%Y-%m-%d", min_value=datetime(2021, 1, 1))
    assert "2021-01-01" == is_datetime_string("%Y-%m-%d", max_value=datetime(2021, 1, 1))
    assert "2021-01-01" == is_datetime_string(
        "%Y-%m-%d",
        min_value=datetime(2021, 1, 1),
        max_value=datetime(2021, 1, 2),
    )
    assert "2021-01-01" != is_datetime_string(
        "%Y-%m-%d",
        min_value=datetime(2021, 1, 2),
        max_value=datetime(2021, 1, 2),
    )



from datetime import datetime

import pytest

from pytest_matchers.matchers import DatetimeString


def test_create():
    matcher = DatetimeString("format")
    assert isinstance(matcher, DatetimeString)
    matcher = DatetimeString("format", min_value=datetime(2021, 1, 1))
    assert isinstance(matcher, DatetimeString)
    matcher = DatetimeString("%Y-%m-%d", min_value="2020-01-01")
    assert isinstance(matcher, DatetimeString)
    matcher = DatetimeString("format", max_value=datetime(2021, 1, 1))
    assert isinstance(matcher, DatetimeString)
    with pytest.raises(
        ValueError,
        match="Invalid datetime string format for max_value: 2020-01-01",
    ):
        DatetimeString("format", max_value="2020-01-01", min_value=datetime(2021, 1, 1))


def test_repr():
    matcher = DatetimeString("format")
    assert repr(matcher) == "To be a datetime string with format 'format'"
    matcher = DatetimeString("%Y-%m-%d", min_value=datetime(2021, 1, 1, 10))
    assert (
        repr(matcher) == "To be a datetime string with format '%Y-%m-%d' "
        "greater or equal than 2021-01-01 00:00:00"
    )
    matcher = DatetimeString("%Y-%m-%d %H:%M:%S", max_value=datetime(2021, 1, 1, 10))
    assert (
        repr(matcher) == "To be a datetime string with format '%Y-%m-%d %H:%M:%S' "
        "lower or equal than 2021-01-01 10:00:00"
    )
    matcher = DatetimeString(
        "%Y-%m-%d",
        min_value=datetime(2021, 1, 1),
        max_value=datetime(2021, 1, 2),
    )
    assert (
        repr(matcher) == "To be a datetime string with format '%Y-%m-%d' "
        "between 2021-01-01 00:00:00 and 2021-01-02 00:00:00"
    )


def test_matches():
    matcher = DatetimeString("format")
    assert matcher != "2021-01-01"
    assert matcher != 3
    assert matcher != datetime(2021, 1, 1)
    matcher = DatetimeString("%Y-%m-%d")
    assert matcher == "2021-01-01"
    assert matcher != "2020-01-01 10:20:15"
    assert matcher != "2020/06/21"
    assert matcher != "10-11-2005"
    assert matcher != datetime(2020, 1, 1)
    matcher = DatetimeString("%Y-%m-%d", min_value=datetime(2021, 1, 1))
    assert matcher == "2021-01-02"
    assert matcher == "2021-01-01"
    assert matcher != "2020-01-01"
    assert matcher != datetime(2020, 1, 1)
    matcher = DatetimeString("%Y-%m-%d", min_value=datetime(2021, 1, 1, 23, 59, 59))
    assert matcher == "2021-01-02"
    assert matcher == "2021-01-01"
    assert matcher != "2020-12-31"
    matcher = DatetimeString("%Y-%m-%d", min_value="2020-01-01")
    assert matcher == "2021-01-01"
    assert matcher != "2019-01-01"
    assert matcher != datetime(2019, 1, 1)
    matcher = DatetimeString("%Y-%m-%d", max_value=datetime(2021, 1, 1))
    assert matcher == "2021-01-01"
    assert matcher != "2022-01-01"
    assert matcher != datetime(2022, 1, 1)
    matcher = DatetimeString(
        "%Y-%m-%d",
        min_value=datetime(2021, 1, 1),
        max_value=datetime(2021, 1, 2),
    )
    assert matcher == "2021-01-01"
    assert matcher == "2021-01-02"
    assert matcher != "2021-01-03"
    assert matcher != datetime(2021, 1, 3)



from pytest_matchers.matchers import Eq, IsInstance
from pytest_matchers.utils.repr_utils import concat_matcher_repr, concat_reprs


def test_concat_matcher_repr():
    matcher = IsInstance(str)
    assert concat_matcher_repr(matcher) == "of 'str' instance"
    matcher = Eq(3)
    assert concat_matcher_repr(matcher) == "to be 3"
    matcher = None
    assert concat_matcher_repr(matcher) is None


def test_concat_reprs():
    assert concat_reprs(None) is None
    assert concat_reprs("I expect") == "I expect"
    assert concat_reprs("I expect", Eq(3)) == "I expect to be 3"
    assert concat_reprs("I expect", Eq(3), Eq(4)) == "I expect to be 3 and to be 4"
    assert (
        concat_reprs("I expect", IsInstance(str), Eq(2), "be happy")
        == "I expect of 'str' instance and to be 2 and be happy"
    )
    assert concat_reprs("I expect", Eq(3), Eq(4), separator="or") == "I expect to be 3 or to be 4"



from pytest_matchers.matchers import EndsWith


def test_create():
    matcher = EndsWith("a")
    assert isinstance(matcher, EndsWith)
    matcher = EndsWith([1, 2])
    assert isinstance(matcher, EndsWith)


def test_repr():
    matcher = EndsWith("a")
    assert repr(matcher) == "To end with 'a'"
    matcher = EndsWith([1, 2])
    assert repr(matcher) == "To end with [1, 2]"


def test_concatenated_repr():
    matcher = EndsWith("a")
    assert matcher.concatenated_repr() == "ending with 'a'"
    matcher = EndsWith([1, 2])
    assert matcher.concatenated_repr() == "ending with [1, 2]"


def test_matches_string():
    matcher = EndsWith("bc")
    assert matcher == "abc"
    assert matcher == "bc"
    assert matcher != "dabcx"
    assert matcher != ["x", "bc", "y"]
    assert matcher != "ac"
    assert matcher != ["abc"]
    assert matcher != 20
    assert matcher != ["x", "bc"]


def test_matches_list():
    matcher = EndsWith([1, 3])
    assert matcher == [2, 1, 3]
    assert matcher == [1, 3]
    assert matcher != [1, 20, 1, 3, 30]
    assert matcher != [10, 20, 30]
    assert matcher != "string"
    assert matcher != 20



# pylint: disable=use-implicit-booleaness-not-comparison
import pytest

from pytest_matchers.matchers import IsList


def test_create():
    matcher = IsList(int)
    assert isinstance(matcher, IsList)

    with pytest.raises(
        ValueError,
        match="Cannot specify length with min_length or max_length",
    ):
        IsList(int, min_length=1, max_length=4, length=2)


def test_repr():
    matcher = IsList()
    assert repr(matcher) == "To be a list"
    matcher = IsList(int)
    assert repr(matcher) == "To be a list of 'int' instance"
    matcher = IsList(str)
    assert repr(matcher) == "To be a list of 'str' instance"
    matcher = IsList(int, length=2)
    assert repr(matcher) == "To be a list of 'int' instance and with length of 2"
    matcher = IsList(int, min_length=2)
    assert repr(matcher) == "To be a list of 'int' instance and with length greater or equal than 2"
    matcher = IsList(int, max_length=2)
    assert repr(matcher) == "To be a list of 'int' instance and with length lower or equal than 2"
    matcher = IsList(int, min_length=2, max_length=4)
    assert repr(matcher) == "To be a list of 'int' instance and with length between 2 and 4"
    matcher = IsList(None, length=2)
    assert repr(matcher) == "To be a list with length of 2"


def test_matches_type():
    assert [] == IsList(int)
    assert IsList(int) == []
    assert [1, 2, 3] == IsList(int)
    assert IsList(int) == [1, 2, 3]
    assert [1, 2, 3] != IsList(str)
    assert [1, 2, 3] != IsList(str)
    assert "string" != IsList(str)


def test_is_list_without_match_type():
    matcher = IsList()
    assert matcher == []
    assert matcher == [1]
    assert matcher == ["a"]
    matcher = IsList(length=2)
    assert matcher == [1, 2]


def test_matches_length():
    matcher = IsList(int, length=2)
    assert matcher == [1, 2]
    assert matcher != [1]
    assert matcher != [1, 2, 3]


def test_matches_min_length():
    matcher = IsList(int, min_length=2)
    assert matcher == [1, 2]
    assert matcher == [1, 2, 3]
    assert matcher != [1]
    assert matcher != []


def test_matches_max_length():
    matcher = IsList(int, max_length=2)
    assert [] == matcher
    assert matcher == []
    assert matcher == [1, 2]
    assert matcher != [1, 2, 3]
    assert matcher != [1, 2, 3, 4]



from pytest_matchers.matchers import (
    Anything,
    Between,
    Contains,
    EndsWith,
    Eq,
    IsInstance,
    Length,
    StartsWith,
)
from pytest_matchers.utils.matcher_utils import (
    as_matcher,
    between_matcher,
    contains_matcher,
    ends_with_matcher,
    is_instance_matcher,
    length_matcher,
    matches_or_none,
    partial_matches_or_none,
    starts_with_matcher,
)


def test_as_matcher():
    matcher = as_matcher(1)
    assert isinstance(matcher, Eq)
    assert matcher == 1
    matcher = as_matcher("string")
    assert isinstance(matcher, Eq)
    assert matcher == "string"
    matcher = as_matcher(Eq(1))
    assert isinstance(matcher, Eq)
    assert matcher == 1
    matcher = as_matcher(Anything())
    assert isinstance(matcher, Anything)
    assert matcher == "something"


def test_matches_or_none():
    assert matches_or_none(None, 1)
    assert matches_or_none(Eq(1), 1)
    assert not matches_or_none(Eq(1), 2)


def test_partial_matches_or_none():
    matcher = Eq(1)
    partial_matcher = partial_matches_or_none(matcher)
    assert partial_matcher(1)
    assert not partial_matcher(2)
    matcher = None
    partial_matcher = partial_matches_or_none(matcher)
    assert partial_matcher(1)
    assert partial_matcher(2)


def test_is_instance_matcher():
    matcher = is_instance_matcher(str)
    assert isinstance(matcher, IsInstance)
    assert matcher == "string"
    matcher = is_instance_matcher(None)
    assert matcher is None


def test_contains_matcher():
    matcher = contains_matcher("string")
    assert isinstance(matcher, Contains)
    assert matcher == "string"
    matcher = contains_matcher(None)
    assert matcher is None


def test_length_matcher():
    matcher = length_matcher(None, 2, 3)
    assert isinstance(matcher, Length)
    assert matcher == "ab"
    matcher = length_matcher(None, None, 3)
    assert isinstance(matcher, Length)
    assert matcher == [1, 2, 3]
    matcher = length_matcher(None, None, None)
    assert matcher is None


def test_starts_with_matcher():
    matcher = starts_with_matcher("string")
    assert isinstance(matcher, StartsWith)
    assert matcher == "string"
    matcher = starts_with_matcher(None)
    assert matcher is None


def test_ends_with_matcher():
    matcher = ends_with_matcher("string")
    assert isinstance(matcher, EndsWith)
    assert matcher == "string"
    matcher = ends_with_matcher(None)
    assert matcher is None


def test_between_matcher():
    matcher = between_matcher(1, 2, None, None, None)
    assert isinstance(matcher, Between)
    assert matcher == 1
    matcher = between_matcher(None, 2, True, None, None)
    assert isinstance(matcher, Between)
    assert matcher == 2
    matcher = between_matcher(1, None, None, False, None)
    assert isinstance(matcher, Between)
    assert matcher == 2
    matcher = between_matcher(None, None, True, None, None)
    assert matcher is None
